"""
This type stub file was generated by pyright.
"""

from ._base import Icon as Icon
from ._base import Menu as Menu
from ._base import MenuItem as MenuItem

def backend() -> Icon:
    """Returns the backend module."""
    ...
