from typing import Any

class KeyringBackend:
    def get_password(self, service: str, username: str) -> str | None: ...
    def set_password(self, service: str, username: str, password: str) -> None: ...
    def delete_password(self, service: str, username: str) -> None: ...
    def with_properties(self, **kwargs: Any) -> KeyringBackend: ...

def get_all_keyring() -> list[KeyringBackend]: ...
