"""
This type stub file was generated by pyright.
"""

import typing

from gi.repository import Gtk
from pystray import _base

if not Gtk.init_check()[0]: ...

def mainloop(f: typing.Any) -> typing.Any:
    """Marks a function to be executed in the main loop.

    The function will be scheduled to be executed later in the mainloop.

    :param callable f: The function to execute. Its return value is discarded.
    """
    ...

class GtkIcon(_base.Icon):
    def __init__(self, *args: typing.Any, **kwargs: typing.Any) -> None: ...
