"""
This type stub file was generated by pyright.
"""

import typing

from ._util.gtk import GtkIcon

class Icon(GtkIcon):
    def __init__(self, *args: typing.Any, **kwargs: typing.Any) -> None: ...
