"""
This type stub file was generated by pyright.
"""

import ctypes

windll = ...
WM_CREATE = ...
WM_DISPLAYCHANGE = ...
WM_NCCREATE = ...
WM_LBUTTONUP = ...
WM_RBUTTONUP = ...
WM_USER = ...
WM_STOP = ...
WM_NOTIFY = ...
LR_DEFAULTSIZE = ...
LR_LOADFROMFILE = ...
MFS_CHECKED = ...
MFS_DEFAULT = ...
MFS_DISABLED = ...
MFS_ENABLED = ...
MFS_GRAYED = ...
MFS_HILITE = ...
MFS_UNCHECKED = ...
MFS_UNHILITE = ...
MFT_BITMAP = ...
MFT_MENUBARBREAK = ...
MFT_MENUBREAK = ...
MFT_OWNERDRAW = ...
MFT_RADIOCHECK = ...
MFT_RIGHTJUSTIFY = ...
MFT_RIGHTORDER = ...
MFT_SEPARATOR = ...
MFT_STRING = ...
MIIM_BITMAP = ...
MIIM_CHECKMARKS = ...
MIIM_DATA = ...
MIIM_FTYPE = ...
MIIM_ID = ...
MIIM_STATE = ...
MIIM_STRING = ...
MIIM_SUBMENU = ...
MIIM_TYPE = ...
MSGFLT_ALLOW = ...
MSGFLT_DISALLOW = ...
MSGFLT_RESET = ...
NIF_MESSAGE = ...
NIF_ICON = ...
NIF_TIP = ...
NIF_STATE = ...
NIF_INFO = ...
NIF_GUID = ...
NIF_REALTIME = ...
NIF_SHOWTIP = ...
NIM_ADD = ...
NIM_MODIFY = ...
NIM_DELETE = ...
NIM_SETFOCUS = ...
NIM_SETVERSION = ...
TPM_CENTERALIGN = ...
TPM_LEFTALIGN = ...
TPM_RIGHTALIGN = ...
TPM_BOTTOMALIGN = ...
TPM_TOPALIGN = ...
TPM_VCENTERALIGN = ...
TPM_NONOTIFY = ...
TPM_RETURNCMD = ...
TPM_LEFTBUTTON = ...
TPM_RIGHTBUTTON = ...
TPM_HORNEGANIMATION = ...
TPM_HORPOSANIMATION = ...
TPM_NOANIMATION = ...
TPM_VERNEGANIMATION = ...
TPM_VERPOSANIMATION = ...
TPM_HORIZONTAL = ...
TPM_VERTICAL = ...
WS_POPUP = ...
PM_NOREMOVE = ...
COLOR_WINDOW = ...
HWND_MESSAGE = ...
IMAGE_ICON = ...
LPMSG = ...
LPPOINT = ...
WNDPROC = ...

class MENUITEMINFO(ctypes.Structure):
    _fields_ = ...

LPMENUITEMINFO = ...

class NOTIFYICONDATAW(ctypes.Structure):
    class VERSION_OR_TIMEOUT(ctypes.Union):
        _fields_ = ...

    class GUID(ctypes.Structure):
        _fields_ = ...

    _fields_ = ...
    _anonymous_ = ...

LPNOTIFYICONDATAW = ...

class TPMPARAMS(ctypes.Structure):
    _fields_ = ...

LPTPMPARAMS = ...

class WNDCLASSEX(ctypes.Structure):
    _fields_ = ...

LPWNDCLASSEX = ...
CreatePopupMenu = ...
CreateWindowEx = ...
DefWindowProc = ...
DestroyIcon = ...
DestroyMenu = ...
DestroyWindow = ...
DispatchMessage = ...
GetCursorPos = ...
GetMessage = ...
GetModuleHandle = ...
InsertMenuItem = ...
LoadImage = ...
PeekMessage = ...
PostMessage = ...
PostQuitMessage = ...
RegisterClassEx = ...
SetForegroundWindow = ...
Shell_NotifyIcon = ...
TranslateMessage = ...
TrackPopupMenuEx = ...
UnregisterClass = ...
RegisterWindowMessage = ...
WM_TASKBARCREATED = ...
ChangeWindowMessageFilterEx = ...
