class KeyringError(Exception): ...
class PasswordSetError(KeyringError): ...
class PasswordDeleteError(KeyringError): ...
class InitError(KeyringError): ...
class KeyringLocked(KeyringError): ...
class NoKeyringError(KeyringError): ...
