"""
This type stub file was generated by pyright.
"""

from typing import Union

from dataclasses import dataclass
from enum import Enum

class AudioSource(Enum):
    """
    Different audios built into Windows
    """

    Default = ...
    IM = ...
    Mail = ...
    Reminder = ...
    SMS = ...
    Alarm = ...
    Alarm2 = ...
    Alarm3 = ...
    Alarm4 = ...
    Alarm5 = ...
    Alarm6 = ...
    Alarm7 = ...
    Alarm8 = ...
    Alarm9 = ...
    Alarm10 = ...
    Call = ...
    Call2 = ...
    Call3 = ...
    Call4 = ...
    Call5 = ...
    Call6 = ...
    Call7 = ...
    Call8 = ...
    Call9 = ...
    Call10 = ...

@dataclass
class ToastAudio:
    """
    Audio configuration in a toast

    :param sound: Selected AudioSource to play
    :type sound: Union[AudioSource, str]
    :param looping: Whether the audio should loop once it ends. Stops abruptly when the notification is dismissed
    :type looping: bool
    :param silent: Silence any audio
    :type silent: bool
    """

    sound: Union[AudioSource, str] = ...
    looping: bool = ...
    silent: bool = ...
    @property
    def sound_value(self) -> str:
        """
        Returns the string value of the selected sound
        """
        ...
