"""
This type stub file was generated by pyright.
"""

import typing

from . import _base

class Icon(_base.Icon):
    _HWND_TO_ICON: typing.Any = ...
    def __init__(self, *args: typing.Any, **kwargs: typing.Any) -> None: ...
    def __del__(self) -> None: ...
