"""
This type stub file was generated by pyright.
"""

import platform

from ._version import __author__
from ._version import __description__
from ._version import __license__
from ._version import __title__
from ._version import __url__
from ._version import __version__
from .events import ToastActivatedEventArgs
from .events import ToastDismissalReason
from .events import ToastDismissedEventArgs
from .events import ToastFailedEventArgs
from .exceptions import InvalidImageException
from .exceptions import ToastNotFoundError
from .exceptions import UnsupportedOSVersionException
from .toast import Toast
from .toast_audio import AudioSource
from .toast_audio import ToastAudio
from .toasters import InteractableWindowsToaster
from .toasters import WindowsToaster
from .wrappers import ToastButton
from .wrappers import ToastButtonColour
from .wrappers import ToastDisplayImage
from .wrappers import ToastDuration
from .wrappers import ToastImage
from .wrappers import ToastImagePosition
from .wrappers import ToastInputSelectionBox
from .wrappers import ToastInputTextBox
from .wrappers import ToastProgressBar
from .wrappers import ToastScenario
from .wrappers import ToastSelection
from .wrappers import ToastSystemButton
from .wrappers import ToastSystemButtonAction

MIN_VERSION = ...
osVersion = int(platform.version().split(".")[2])
if platform.system() == "Windows" and osVersion < MIN_VERSION: ...
__all__ = [
    "__author__",
    "__description__",
    "__license__",
    "__title__",
    "__url__",
    "__version__",
    "ToastActivatedEventArgs",
    "ToastDismissalReason",
    "ToastDismissedEventArgs",
    "ToastFailedEventArgs",
    "InvalidImageException",
    "ToastNotFoundError",
    "UnsupportedOSVersionException",
    "AudioSource",
    "ToastAudio",
    "Toast",
    "InteractableWindowsToaster",
    "WindowsToaster",
    "ToastButton",
    "ToastButtonColour",
    "ToastDisplayImage",
    "ToastDuration",
    "ToastImage",
    "ToastImagePosition",
    "ToastInputSelectionBox",
    "ToastInputTextBox",
    "ToastProgressBar",
    "ToastScenario",
    "ToastSelection",
    "ToastSystemButton",
    "ToastSystemButtonAction",
]
