"""
This type stub file was generated by pyright.
"""

DESTINATION: str = ...
PATH: str = ...

class Notifier:
    def __init__(self) -> None: ...
    def notify(self, title: str, message: str, icon: str) -> None:
        """Displays a notification message.

        :param str title: The message title.

        :param str message: The actual message.

        :param str icon: The icon path.
        """
        ...

    def hide(self) -> None:
        """Hides the notification displayed by :meth:`notify`."""
        ...
